class EventManager {
	constructor() {
		this.emitters = new Map(); // Stores emitter names and their associated listeners
		this.state = {}; // Global state of all emitters
	}

	registerEmitter(
		emitter,
		eventName,
		emitterName,
		detailFn = () => null,
		initialState = null,
		manageState = true
	) {
		if (!emitter || !eventName || !emitterName || !detailFn) {
			throw new Error(
				'Emitter, event name, emitter name, and detail function must be provided'
			);
		}

		// Initialize listeners array for this emitter if it doesn't exist
		if (!this.emitters.has(emitterName)) {
			this.emitters.set(emitterName, {
				emitter,
				eventName,
				listeners: [],
				detailFn,
				handler: null,
				manageState,
			});
		}

		// Initialize the state with the initial state value if provided and state management is enabled
		if (initialState !== null && manageState) {
			this._updateState(emitterName, initialState);
		}

		const handler = (e) => {
			const detail = detailFn(e);
			if (manageState) {
				this._triggerCustomEvent(emitterName, detail, emitter);
			} else {
				// Just trigger the custom event without managing state
				this._notifyListeners(emitterName, detail);
			}
		};

		// Add event listener to the emitter
		if (emitter.listen && typeof emitter.listen === 'function') {
			emitter.listen(eventName, handler);
		} else {
			emitter.addEventListener(eventName, handler);
		}

		// Store the handler in the emitter map for potential removal later
		this.emitters.get(emitterName).handler = handler;
	}

	unregisterEmitter(emitterName) {
		if (!this.emitters.has(emitterName)) {
			return;
		}

		const { emitter, eventName, handler } = this.emitters.get(emitterName);

		if (emitter.unlisten && typeof emitter.unlisten === 'function') {
			// Unlisten for MDCSelect and similar MDC components
			emitter.unlisten(eventName, handler);
		} else {
			// For standard DOM elements
			emitter.removeEventListener(eventName, handler);
		}

		// Remove the emitter from the map
		this.emitters.delete(emitterName);
	}

	destroyEmitter(emitterName) {
		if (!this.emitters.has(emitterName)) {
			return;
		}

		const { emitter } = this.emitters.get(emitterName);

		if (emitter.destroy && typeof emitter.destroy === 'function') {
			// Destroy the MDCSelect component
			emitter.destroy();
		}

		// Remove the emitter from the map
		this.emitters.delete(emitterName);
	}

	registerListener(emitterName, listenerFn) {
		if (!this.emitters.has(emitterName)) {
			throw new Error(
				`Emitter with name ${emitterName} is not registered`
			);
		}

		// Add the listener function to the emitter's listener array
		this.emitters.get(emitterName).listeners.push(listenerFn);
	}

	unregisterListener(emitterName, listenerFn) {
		if (!this.emitters.has(emitterName)) {
			throw new Error(
				`Emitter with name ${emitterName} is not registered`
			);
		}

		const { listeners } = this.emitters.get(emitterName);
		const index = listeners.indexOf(listenerFn);

		if (index !== -1) {
			listeners.splice(index, 1);
		}
	}

	setState(emitterName, newState) {
		if (!this.emitters.has(emitterName)) {
			throw new Error(
				`Emitter with name ${emitterName} is not registered`
			);
		}

		if (this.emitters.get(emitterName).manageState) {
			this._updateState(emitterName, newState);
			this._notifyListeners(emitterName);
		}
	}

	_triggerCustomEvent(emitterName, detail = {}, emitter) {
		if (!this.emitters.has(emitterName)) {
			throw new Error(
				`Emitter with name ${emitterName} is not registered`
			);
		}

		if (emitter.emit && typeof emitter.emit === 'function') {
			// For MDCSelect and similar MDC components
			emitter.emit(`${emitterName}:event`, detail);
		} else {
			// For standard DOM elements
			const customEvent = new CustomEvent(`${emitterName}:event`, {
				detail,
			});
			emitter.dispatchEvent(customEvent);
		}

		// Update the state and notify all listeners
		this._updateState(emitterName, detail);
		this._notifyListeners(emitterName);
	}

	_updateState(emitterName, detail) {
		if (this.emitters.get(emitterName).manageState) {
			this.state[emitterName] = detail;
		}
	}

	_notifyListeners(emitterName, detail = null) {
		const { listeners, manageState } = this.emitters.get(emitterName);
		listeners.forEach((listenerFn) => {
			if (manageState) {
				listenerFn.call(null, this.state);
			} else {
				listenerFn.call(null, detail);
			}
		});
	}

	getState() {
		return this.state;
	}

	getEmitter(emitterName) {
		return this.emitters.get(emitterName);
	}
}
